from Tkinter import *

class DrawingWindow:

    def __init__(self, windowWidth, windowHeight, xMin, xMax, yMin, yMax):
        # Make a window to draw on
        top = Tk()
        self.windowWidth = windowWidth
        self.windowHeight = windowHeight
        self.w = Canvas(top, width=self.windowWidth, height=self.windowHeight)
        self.w.pack()
        
        # multiply an input value by this to get pixels
        self.xScale = windowWidth / float(xMax - xMin)
        self.yScale = windowHeight / float(yMax - yMin)

        self.xMin = xMin
        self.yMin = yMin
        self.xMax = xMax
        self.yMax = yMax

    def scaleX(self, x):
        return self.xScale * (x - self.xMin)
    
    def scaleY(self, y):
        return self.windowHeight - self.yScale * (y - self.yMin)
        
    def drawPoint(self, x, y, color = "blue"):
        windowX = self.scaleX(x)
        windowY = self.scaleY(y)
        return self.w.create_rectangle(windowX-1, windowY-1,
                                       windowX+1, windowY+1,
                                       fill = color, outline = color)

    def drawRobot(self, x, y, noseX, noseY,color = "blue"):
        windowX = self.scaleX(x)
        windowY = self.scaleY(y)
        return (self.w.create_rectangle(windowX-2, windowY-2,
                                        windowX+2, windowY+2,
                                        fill = color, outline = color),
                self.w.create_line(windowX, windowY,
                                   self.scaleX(noseX),self.scaleY(noseY),
                                   fill=color))

    def drawRect(self, (x1,y1), (x2,y2), color = "black"):
        self.w.create_rectangle(self.scaleX(x1), self.scaleY(y1),
                                self.scaleX(x2), self.scaleY(y2),
                                fill = color, outline = "black")

    def drawLineSeg(self, x1, y1, x2, y2, color = "black"):
        return self.w.create_line(self.scaleX(x1),self.scaleY(y1),
                                  self.scaleX(x2),self.scaleY(y2),
                                  fill = color)

    def drawLine(self, (a,b,c), color = "black"):
        startX = self.scaleX(self.xMin)
        startY = self.scaleY(- (a * self.xMin + c) / b)
        endX = self.scaleX(self.xMax)
        endY = self.scaleY(- (a * self.xMax + c) / b)
        return self.w.create_line(startX, startY, endX, endY, fill = color)

    def delete(self, thing):
        self.w.delete(thing)
