import DrawingWindowOld
from DrawingWindowOld import *
reload(DrawingWindowOld)

import random

class World:
    colors = {0: 'white', 1: 'black', 2: 'red', 3:'green', 4:'blue'}
    inverseColors = {'white':0, 'black':1, 'red':2, 'green':3, 'blue':4}

    # obstacles, red, green, and blue are lists of points (pairs of indices)
    # robot is a pair of indices
    def __init__(self, xdim, ydim, black, red, green, blue, robot,
                 sensorModel, motionModel, verbose = True):
        self.xdim = xdim
        self.ydim = ydim
        self.sensorModel = sensorModel
        self.motionModel = motionModel
        self.verbose = verbose
        self.grid = zeros2Int(xdim, ydim)
        for (x,y) in black:
            self.grid[x][y] = World.inverseColors['black']
        for (x,y) in red:
            self.grid[x][y] = World.inverseColors['red']
        for (x,y) in green:
            self.grid[x][y] = World.inverseColors['green']
        for (x,y) in blue:
            self.grid[x][y] = World.inverseColors['blue']

        self.robotLoc = robot
        # Leave a margin of 0.2 around the edges of the window
        self.window = DrawingWindow(xdim*50, ydim*50+10, -0.2, xdim+0.2, \
                                    -0.2, ydim + 0.2)

        self.belief = arrayDiv(ones2Float(xdim, ydim),(xdim * ydim))
        self.printBelief()
        self.draw()

    def draw(self):
        for x in range(self.xdim):
            for y in range(self.ydim):
                self.window.drawRect((x, y), (x+1, y+1),
                                     color = World.colors[self.grid[x][y]])
                self.window.drawRect((x + 0.2, y + 0.2), (x + 0.8, y + 0.8),
                                     color = beliefColor(self.belief[x][y]))
        (rx,ry) = self.robotLoc
        self.window.drawRect((rx + 0.4, ry+0.4), (rx + 0.6, ry + 0.6),
                             color = 'orange')
    def printBelief(self):
        for y in range(self.ydim-1, -1, -1):
            string = ""
            for x in range(self.xdim):
                string +=  "%8.4f"%self.belief[x][y]
            print string

    def actionUpdate(self, (ax, ay)):
        # Just makes a list of lists of floating point zeros
        newbelief = zeros2Float(self.xdim, self.ydim)
        # For each old belief state, move the probability mass
        # according to the action.  Use self.motionModel to find the
        # non-zero-probability transitions.
        # self.belief[x][y] is the current belief that the robot is at
        # location (x,y)

        # YOUR CODE HERE

        self.belief = newbelief
        if self.verbose:
            print "Updated belief after action", (ax, ay)
            self.printBelief()

    def observationUpdate(self, obs):
        # Use self.sensorModel.  Note that
        # self.sensorModel[self.grid[x][y]][o]
        # is the probability of making observation o given that the
        # robot is in location (x,y)

        # YOUR CODE HERE

        if self.verbose:
            print "Updated belief after observation", obs
            self.printBelief()

    def move(self, action):
        # Move robot in simulator
        stateDist = self.motionModel(self.robotLoc, action, self.xdim,
                                     self.ydim)
        self.robotLoc = multinomialListDraw(stateDist)
        
        # Update belief state based on intended motion
        self.actionUpdate(action)
        
        # Get an observation.
        nominalObs = self.grid[self.robotLoc[0]][self.robotLoc[1]]
        obsDist = self.sensorModel[nominalObs]
        actualObs = multinomialDraw(obsDist)
        
        #Update belief state based on observation
        self.observationUpdate(actualObs)
        self.draw()
        
    def north(self): self.move((0,1))
    def south(self): self.move((0,-1))
    def east(self): self.move((1,0))
    def west(self): self.move((-1,0))
    def stay(self): self.move((0,0))

    # Set the belief state back to uniform
    def reset(self):
        self.belief = arrayDiv(ones2Float(self.xdim,self.ydim),
                               self.xdim*self.ydim)
        self.draw()

##########################################################################
###
###     Helper functions                
###     
##########################################################################

# Make a gray-scale color by scaling the probability to be between 0
# and 255, and then making three copies of it in hex in a string
def beliefColor(p):
    cval = int(p*255)
    cval = 55+int(p*200)
    return "#%02x%02x%02x"%(cval, cval, cval)

def clip(v, vmin, vmax):
    return max(vmin, min(vmax, v))

# Distribution is a list of numbers that sum to 1, encoding a
# multinomial probability distribution.  Generate a random draw from
# that distribution
def multinomialDraw(dist):
    r = random.random()
    sum = 0.0
    for i in range(len(dist)):
        sum += dist[i]
        if r < sum:
            return i

# Distribution is a list of pairs, the second elements of which
# numbers that sum to 1, encoding a multinomial probability
# distribution.  Generate a random draw from that distribution
def multinomialListDraw(dist):
    r = random.random()
    sum = 0.0
    for i in range(len(dist)):
        sum += dist[i][1]
        if r < sum:
            return dist[i][0]

##########################################################################
###
###     So we don't need Numeric
###     
##########################################################################
def makeArray(xdim, ydim, init):
    result = []
    for x in range(xdim):
        result = result + [[init]*ydim]
    return result

def ones2Float(xdim, ydim):
    return makeArray(xdim, ydim, 1.0)

def zeros2Float(xdim, ydim):
    return makeArray(xdim, ydim, 0.0)

def zeros2Int(xdim, ydim):
    return makeArray(xdim, ydim, 0)

def arrayDiv(arr, x):
    return map(lambda row: map(lambda elt: elt / x, row), arr)
